﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using System.Web.Mvc;
using KendoMVCWrappers.Models;
using Kendo.Mvc.UI;
using Kendo.Mvc.Extensions;

namespace KendoMVCWrappers.Controllers
{
    public class HomeController : Controller
    {
        public static List<Person> people = new List<Person>();

        static HomeController()
        {
            people.Add(new Person { PersonID = 1, Name = "John", BirthDate = new DateTime(1968, 6, 26) });
            people.Add(new Person { PersonID = 2, Name = "Sara", BirthDate = new DateTime(1974, 9, 13) });
        }

        public ActionResult Index()
        {
            ViewBag.Message = "Modify this template to kick-start your ASP.NET MVC application.";

            return View();
        }

        public ActionResult GetPeople([DataSourceRequest] DataSourceRequest dsRequest)
        {
            var result = people.ToDataSourceResult(dsRequest);
            return Json(result);
        }

        public ActionResult UpdatePerson([DataSourceRequest] DataSourceRequest dsRequest, [Bind(Prefix="models")]List<Person> updated)
        {
            if (people != null && ModelState.IsValid)
            {
                foreach (var person in updated)
                {
                    var toUpdate = people.FirstOrDefault(p => p.PersonID == person.PersonID);
                    toUpdate.Name = person.Name;
                    toUpdate.BirthDate = person.BirthDate;
                }
            }

            return Json(updated.ToDataSourceResult(dsRequest, ModelState));
        }

        public ActionResult CreatePerson([DataSourceRequest] DataSourceRequest dsRequest, [Bind(Prefix = "models")]List<Person> created)
        {
            if (people != null && ModelState.IsValid)
            {
                foreach (var person in created)
                {
                    person.PersonID = people.Count + 1;
                    people.Add(person);
                }
            }

            return Json(created.ToDataSourceResult(dsRequest, ModelState));
        }

        public ActionResult About()
        {
            ViewBag.Message = "Your quintessential app description page.";

            return View();
        }

        public ActionResult ChooseLanguage(String languageCode)
        {
            try
            {
                if (LocalizationHelper.IsValidLanguageCode(languageCode))
                    Session[SessionKeyNames.LANGUAGE] = languageCode;

                Thread.CurrentThread.CurrentUICulture = LocalizationHelper.GetCultureInfo(languageCode);
                Thread.CurrentThread.CurrentCulture = LocalizationHelper.GetCultureInfo(languageCode);

                return RedirectToAction("Index", "Home");
            }
            catch (Exception)
            {
                throw;
            }
        }
    }

    public static class LanguageCodes
    {
        public const String GERMAN = "de";
        public const String ENGLISH = "en";
    }

    public static class SessionKeyNames
    {
        public const String LANGUAGE = "Language";
    }

    public static class LocalizationHelper
    {
        /// <summary>
        /// List of all currently supported language codes.
        /// </summary>
        private static List<String> ValidLanguageCodes;

        static LocalizationHelper()
        {
            ValidLanguageCodes = new List<String>();
            ValidLanguageCodes.Add(LanguageCodes.GERMAN);
            ValidLanguageCodes.Add(LanguageCodes.ENGLISH);
        }

        /// <summary>
        /// Returns the language code for the default language.
        /// </summary>
        /// <remarks>The current default language is 'English'.</remarks>
        public static String GetDefaultLanguageCode()
        {
            try
            {
                return LanguageCodes.ENGLISH;
            }
            catch (Exception)
            {
                throw;
            }
        }

        /// <summary>
        /// Checks whether a given language code is a valid one (the check is performed case-insensitive).
        /// </summary>
        /// <param name="languageCode">The language code to check.</param>
        /// <returns>True ... valid / False ... not valid</returns>
        public static Boolean IsValidLanguageCode(String languageCode)
        {
            try
            {
                if (!String.IsNullOrEmpty(languageCode) && (ValidLanguageCodes.Count(item => String.Compare(item, languageCode, true) == 0) > 0))
                    return true;
                else
                    return false;
            }
            catch (Exception)
            {
                throw;
            }
        }

        /// <summary>
        /// Creates and returns a CultureInfo object for a given language code.
        /// </summary>
        /// <param name="languageCode">The language code for which to create a cultureInfo.</param>
        /// <remarks>If the language code is invalid, an exception is thrown.</remarks>
        public static CultureInfo GetCultureInfo(String languageCode)
        {
            try
            {
                if (IsValidLanguageCode(languageCode))
                {
                    // Get the language code
                    String languageCodeCorrect = ValidLanguageCodes.First(item => String.Compare(item, languageCode, true) == 0);
                    return CultureInfo.CreateSpecificCulture(languageCodeCorrect);
                }
                else
                    throw new NotSupportedException(String.Format("Language '{0}' not supported", languageCode));
            }
            catch (Exception)
            {
                throw;
            }
        }

    }
}
